/** 
 * bleifrei utility library
 *
 * Copyright (C) 2003 Ulf Borgenstam, Jonas Svensson.
 *
 * Do not distribute. If you know someone interested in obtaining these sources
 * please have them contact us.
 */

// includes
#include <iostream>
#include <list>
#include "../platform/windows.hpp"

#include "Console.hpp"

using namespace std;

namespace bleifrei {

	io::Console Singleton<io::Console>::instance;

	namespace io {

		bool
		Console::init(int maxLines)
		{
			this->maxLines = maxLines;

			AllocConsole();
			SetConsoleTitle("Debug output");
			freopen("CONIN$","rb",stdin);
			freopen("CONOUT$","wb",stdout);
			freopen("CONOUT$","wb",stderr);

			return true;
		}

		void
		Console::destroy(void)
		{
			FreeConsole();
		}

		void Console::printf(float r, float g, float b, const char *str, ...)
		{
			char text[1024];

			if (str == NULL) {
				print(r, g, b, "");
				return;
			}

			va_list	ap;
			va_start(ap, str);
			vsprintf(text, str, ap);
			va_end(ap);

			print(r, g, b, text);
		}

		void Console::print(float r, float g, float b, const string &str)
		{
			lines.push_back(Line(r, g, b, str));
			while (getLineCount() >= maxLines) lines.pop_back();
			setConsoleColor(r, g, b);
			cout << str << endl;
		}

		void
		Console::setConsoleColor(float r, float g, float b)
		{
			unsigned short attrib = (r > 0.5 || g > 0.5 || b > 0.5) ? FOREGROUND_INTENSITY : 0;
			if (r > 0.25f) attrib |= FOREGROUND_RED;
			if (g > 0.25f) attrib |= FOREGROUND_GREEN;
			if (b > 0.25f) attrib |= FOREGROUND_BLUE;

			SetConsoleTextAttribute(GetStdHandle(STD_ERROR_HANDLE), attrib);
		}

	}
}
