
// includes
#include "../input/Mouse.hpp"
#include "../input/Keyboard.hpp"
#include "System.hpp"
#include "Window.hpp"

bool startApplication(void);
void endApplication(void);

using namespace bleifrei::input;

namespace bleifrei {

	platform::System Singleton<platform::System>::instance;

	namespace platform {

		bool
		System::init(void)
		{
			m_frameTime = 0;	
			m_timer.reset();

			return true;
		}

		void
		System::destroy(void)
		{
		}

		void
		System::resetTimer(void)
		{
			m_frameTime = 0;
			m_timer.reset();
		}

		bool
		System::update(void)
		{
			m_frameTime = m_timer.getTimeAndReset() * 0.001f;

			MSG msg;

			if (PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE)) {
				if (msg.message == WM_QUIT)	{
					return false;
				}
				else {
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}
			}
			return true;
		}

		LRESULT
		CALLBACK
		System::windows_callback(HWND windowHandle, UINT message, WPARAM wParam, LPARAM lParam)
		{
			switch(message) {

			case WM_KEYDOWN:
				break;

			case WM_CHAR:
				Keyboard::getInstance().keyTypedEvent((char)wParam);
				break;

			case WM_CREATE:
				break;

			case WM_CLOSE:
				Window::getInstance().closeEvent();
				return 0;
				break;

			case WM_MOUSEWHEEL:
				Mouse::getInstance().scrollWheel(GET_WHEEL_DELTA_WPARAM(wParam));
				break;
			
			case WM_SIZE:
			{
				Window::getInstance().resizeEvent(LOWORD(lParam), HIWORD(lParam));
				break;
			}

			default:
				break;
			}

			return (DefWindowProc(windowHandle, message, wParam, lParam));
		}

	}
}

using namespace bleifrei::platform;

void applicationMain(void);

int
WINAPI
WinMain(HINSTANCE instance, HINSTANCE prevInstance, char *lpzCommandLine, int iShow)
{
	System::getInstance().windows_setInstanceHandle(instance);

	applicationMain();
	return 0;
}

