
#include "../gfx/opengl.hpp"
// includes
#include <fstream>

#include "../img/Image.hpp"
#include "../img/TGAImage.hpp"

#include "TextureManager.hpp"
#include "Texture.hpp"

using namespace bleifrei::img;
using namespace std;

namespace bleifrei {

	tex::TextureManager Singleton<tex::TextureManager>::instance;

	namespace tex {

		TextureManager::TextureManager(void)
		{
		}

		TextureManager::~TextureManager(void)
		{
			destroy();
		}

		bool
		TextureManager::init(const string &path, bool deferredLoading)
		{
			this->m_texturePath = path;
			m_deferredLoading = deferredLoading;
			
			size_t pathLength = path.length();
			if (pathLength)
			{
				char last = path.at(pathLength - 1);

				if (last != '\\' && last != '/')
				{
					m_texturePath += '/';
				}
			}
			return true;
		}

		void
		TextureManager::destroy(void)
		{
			map<string, Texture *>::iterator it;
			for (it = m_textures.begin(); it != m_textures.end(); it++)
			{
				if (it->second) it->second->release();
			}
			m_textures.clear();
		}

		Texture *
		TextureManager::load(const string &filename)
		{
			if (filename.empty()) return 0;

			Texture *texture;

			map<string, Texture *>::iterator it = m_textures.find(filename);

			if (it != m_textures.end())
			{
				texture = it->second;
			}
			else
			{
				Texture2 *texture_2d = new Texture2();
				texture_2d->setAnisotropicFiltering(8);
				if (!m_deferredLoading)
				{
					// $TODO: relocate to function with error-handling, pronto!
					TGAImage image(ifstream((m_texturePath + filename).c_str(), ios::binary));
					texture_2d->load(&image);
				}
				texture = texture_2d;

				m_textures.insert(map<string, Texture *>::value_type(filename, texture));
			}

			return texture;
		}

		void
		TextureManager::performDeferredLoading(void)
		{
			if (m_deferredLoading)
			{
				reloadAll();
			}
		}

		void
		TextureManager::reloadAll(void)
		{
			map<string, Texture *>::iterator it;
			for (it = m_textures.begin(); it != m_textures.end(); ++it)
			{
				// $TODO: relocate to function with error-handling, pronto!
				TGAImage image(ifstream((m_texturePath + it->first).c_str(), ios::binary));
				((Texture2 *)it->second)->load(&image);
			}
		}

		string
		TextureManager::getTextureName(Texture *texture)
		{
			map<string, Texture *>::iterator it;
			for (it = m_textures.begin(); it != m_textures.end(); ++it)
			{
				if (texture == it->second)
				{
					return it->first;
				}
			}	

			return "";
		}



	}
}