

// include
#include <fstream>
#include <string>

#include "bleifrei/gfx/EffectManager.hpp"
#include "bleifrei/gfx/Font.hpp"
#include "bleifrei/gfx/opengl.hpp"
#include "bleifrei/io/ConfigFile.hpp"
#include "bleifrei/io/Console.hpp"
#include "bleifrei/platform/System.hpp"
#include "bleifrei/tex/TextureManager.hpp"

#include "Button.hpp"
#include "ConsoleOutput.hpp"
#include "Frame.hpp"
#include "FXWindow.hpp"
#include "Label.hpp"
#include "MousePointer.hpp"
#include "Toolbar.hpp"
#include "ProgressIndicator.hpp"
#include "ScrollBar.hpp"
#include "Window.hpp"
#include "GUI.hpp"

#include "../global.hpp"

using namespace bleifrei::gfx;
using namespace bleifrei::gui;
using namespace bleifrei::io;
using namespace bleifrei::math;
using namespace bleifrei::tex;
using namespace bleifrei;
using namespace std;

void
reloadTexturesCallback(Button *)
{
	g_textureManager.reloadAll();
}

FXWindow *effectWindow = 0;
void
reloadEffectCallback(Button *)
{
	Console::getInstance().getLines().clear();
	initEffect();
	effectWindow->setEffect(g_effectManager.reload("postprocessing.fx"));
}

void
togglePostprocessing(Button *)
{
	if (effectWindow->isVisible())
		effectWindow->hide();
	else
		effectWindow->show();
}

void
quitCallback(Button *)
{
	g_system.shutdown();
}

Window *sliderWindow = 0;
void
toggleSliders(Button *)
{
	if (sliderWindow->isVisible())
		sliderWindow->hide();
	else
		sliderWindow->show();
}

GUI::GUI(void)
{
	m_font = new Font(ConfigFile(ifstream("kodden.fnt")));

	m_loadingFrame = new Frame();
	m_mainFrame = new Frame();
	Frame *toolbarAlignRightFrame = 0;

	Button *reloadTexturesButton = 0;
	Button *reloadEffectButton = 0;
	Button *togglePostprocessingButton = 0;
	Button *quitButton = 0;

	add(m_mainFrame);

	m_mainFrame->add(effectWindow = new FXWindow(g_effectManager.load("postprocessing.fx"), m_font, 200, 200, 40, 40));
	if (!g_postprocessing) effectWindow->hide();

	m_mainFrame->add(m_toolbar = new Toolbar("toolbar.tga", g_width, 32));
	m_toolbar->add(new Label(Vector3f(192 / 255.0f, 192 / 255.0f, 192 / 255.0f), "\177", m_font, 15, 7, 8, 8));
	//m_toolbar->add(reloadTexturesButton = new Button("Reload textures", m_font, 33, 3, 150, 15));
	m_toolbar->add(reloadEffectButton = new Button("Reload .fx", m_font, 33, 3, 150, 15));
	m_toolbar->add(togglePostprocessingButton = new Button("Toggle Post-processing", m_font, reloadEffectButton->getX() + reloadEffectButton->getWidth() + 5, 3, 150, 15));
	quitButton = new Button("Quit", m_font, 0, 3, 150, 15);
	m_toolbar->add(toolbarAlignRightFrame = new Frame(GUIComponentFlagAlignRight, quitButton->getWidth() + 10, 22, 0, 0));
	toolbarAlignRightFrame->add(quitButton);

	m_mainFrame->add(new ConsoleOutput(m_font, 2, 32, 0, 0));

	m_mainFrame->add(sliderWindow = new Window(200, 400, 0, 22));
	sliderWindow->setFlag(GUIComponentFlagAlignRight);
	sliderWindow->add(new Label(Vector3f(192 / 255.0f, 192 / 255.0f, 192 / 255.0f), "Parameters", m_font, 24, 24, 0, 8));
	sliderWindow->add(new Label(Vector3f(192 / 255.0f, 192 / 255.0f, 192 / 255.0f), "test:", m_font, 32, 37, 0, 8));
	sliderWindow->add(new ScrollBar(32, 69, 136, 16));
	sliderWindow->add(new ScrollBar(32, 87, 136, 16));
	sliderWindow->add(new ScrollBar(32, 105, 136, 16));
	sliderWindow->hide();

	//reloadTexturesButton->setCallback(reloadTexturesCallback);
	reloadEffectButton->setCallback(reloadEffectCallback);
	togglePostprocessingButton->setCallback(togglePostprocessing);
	quitButton->setCallback(quitCallback);

	Frame *loadingbarFrame;
	add(m_loadingFrame);
	m_loadingFrame->add(loadingbarFrame = new Frame(GUIComponentFlagAlignRight | GUIComponentFlagAlignBottom, 104 + 2, 14 + 2));
	loadingbarFrame->add(m_loadingbar = new ProgressIndicator(0, 8, 104, 6));
	loadingbarFrame->add(new Label(Vector3f(192 / 255.0f, 192 / 255.0f, 192 / 255.0f), "Loading...", m_font, 0, 0, 104, 8));
	m_loadingFrame->add(m_actionLabel = new Label(Vector3f(192 / 255.0f, 192 / 255.0f, 192 / 255.0f), "", m_font, 2, 0, 0, 10));

	m_actionLabel->setFlags(GUIComponentFlagAlignCenter);

	m_mousePointer = new MousePointer("mouse.tga", m_font);

	setState(GUIStateMain);
	container.resize(g_width, g_height);
}

GUI::~GUI(void)
{
	delete m_mousePointer;
	delete m_font;
}

void
GUI::render(void)
{
	glDisable(GL_DEPTH_TEST);
	enableOrtho(g_width, g_height);
	{
		GUIManager::render();

		m_mousePointer->render();
	}
	disableOrtho();
	glEnable(GL_DEPTH_TEST);
}

void
GUI::showCursor(void)
{
	m_mousePointer->show();
}

void
GUI::hideCursor(void)
{
	m_mousePointer->hide();
}

void
GUI::setState(GUIState state)
{
	switch (state)
	{
	case GUIStateLoading :
		m_mousePointer->hide();
		m_loadingFrame->show();
		m_mainFrame->hide();
		break;
	case GUIStateMain :
		m_mousePointer->show();
		m_loadingFrame->hide();
		m_mainFrame->show();
		break;
	}
}

void
GUI::setLoadingProgressMax(int max)
{
	m_loadingbar->setMax(max);
}

void
GUI::setLoadingProgress(int progress)
{
	m_loadingbar->setValue(progress);
}

void
GUI::setLoadingText(const string &text)
{
	m_actionLabel->setText(text);
}

// enables orthogonal projection
void
GUI::enableOrtho(int width, int height) 
{
	glPushMatrix();
	glLoadIdentity();

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();

	glLoadIdentity();
	glOrtho(0.0f, width, height, 0.0f, -1.0f, 1.0f);

	glMatrixMode(GL_MODELVIEW);
}

// resets perspective projection
void
GUI::disableOrtho(void)
{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}